const fp = require("lodash/fp");
const utils = require("./utils");

const listModules = fp.flow(utils.listRequiredModules, fp.keyBy("exported"));

const rootModuleName = "material-ui/index.es";

const settings = {
  index: "material-ui",
  modules: listModules(rootModuleName)
};

module.exports = babel => ({
  visitor: {
    ImportDeclaration(declaration) {
      const specifiers = declaration.node.specifiers;

      if (specifiers.length === 0) {
        return;
      }

      if (declaration.node.source.value !== settings.index) {
        return;
      }

      const types = babel.types;
      const importSpecifiers = specifiers.filter(spec =>
        types.isImportSpecifier(spec)
      );

      if (importSpecifiers.length !== specifiers.length) {
        return;
      }

      importSpecifiers.forEach(spec => {
        const name = spec.imported.name;
        const moduleSettings = settings.modules[name];

        if (!moduleSettings) {
          throw new Error(
            `${settings.index} does not contain module "${name}"`
          );
        }

        declaration.insertBefore(
          types.importDeclaration(
            [
              types.importSpecifier(
                types.identifier(spec.local.name),
                types.identifier(moduleSettings.local)
              )
            ],
            types.stringLiteral(moduleSettings.source)
          )
        );
      });

      declaration.remove();
    }
  }
});
