const fp = require("lodash/fp");
const fs = require("fs");
const path = require("path");
const Module = require("module");
const babylon = require("babylon");

const createImportMapper = moduleDir =>
  fp.flow(
    fp.get("program.body"),
    fp.filter(x => x.type === "ExportNamedDeclaration"),
    fp.flatMap(node =>
      node.specifiers.map(specifier => ({
        local: specifier.local.name,
        exported: specifier.exported.name,

        source: path.posix.join(moduleDir, node.source.value)
      }))
    )
  );

const listRequiredModules = packageModuleFile => {
  try {
    const module = new Module();

    // eslint-disable-next-line no-underscore-dangle
    module.paths = Module._nodeModulePaths(process.cwd());

    // eslint-disable-next-line no-underscore-dangle
    const moduleFilePath = Module._resolveFilename(packageModuleFile, module);
    const packageModuleDir = path.dirname(packageModuleFile);
    const moduleFile = fs.readFileSync(moduleFilePath, "utf-8");
    const astMapper = createImportMapper(packageModuleDir);

    const ast = babylon.parse(moduleFile, { sourceType: "module" });

    return astMapper(ast);
  } catch (e) {
    if (e.code !== "MODULE_NOT_FOUND") {
      throw e;
    }

    console.warn(e.message);

    return [];
  }
};

module.exports = { listRequiredModules };
